// ======================================================================
// creep.hpp - creeping function (local optimization) for the GA Solver
// 
// 102500: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 110500: Benjamin Han <benhdj@cs.cmu.edu> Bug fix: the old score of
//         a sentence depends on the particular solution - now it's obtained
//         from EvalFunc::()
// ======================================================================

//   Copyright (C) 2000 Benjamin Han <benhdj@cs.cmu.edu>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef CREEP_HPP
#define CREEP_HPP

#include "dict.hpp"
#include "solver.hpp"
#include "eval.hpp"

struct Trans
{
  int sid;   // source word ID
  int tid;   // target word ID

  Trans (int sid, int tid):sid(sid),tid(tid) {}
};

typedef list<Trans> TList;

struct DFSNode
{
  int wid;                    // translation word ID
  Sentence::size_type depth;  // depth of the DFS tree
  TagMap tMap;                // part-of-speech distribution so far
  TList tList;                // new translation
  double lScore;              // locality score so far (from the root node)
  int k;                      // used for normalizing lScore
};

class CreepFunc: public SolCreepFunc
{
  const Dict &d1, &d2;
  EvalFunc &ef;

  // 'sid' is source languge word ID and 'tid' is target language ID;
  // 's' is the sentence in the target language - used to compute locality
  // score
  void initDFSNode (int sid, int tid, DFSNode &parent, DFSNode &child,
		    const Sentence &s);

public:

  CreepFunc (const Dict &d1, const Dict &d2, EvalFunc &ef):d1(d1),d2(d2),ef(ef) {}

  bool operator () (Solution &sol);
};

#endif
