// ======================================================================
// eval.hpp - global evaluation of a solution for GA Solver
// 
// 101900: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 110500: Benjamin Han <benhdj@cs.cmu.edu> ScoreTable is added to
//         facilitate incremental, 'smart' evaluation
// 090301: Benjamin Han <benhdj@cs.cmu.edu> Minor revisions to supress warnings
//         under -pedantic -Wall.
// ======================================================================

//   Copyright (C) 2000 Benjamin Han <benhdj@cs.cmu.edu>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


#ifndef EVAL_HPP
#define EVAL_HPP

#include <vector>
#include <algorithm>

#include "dict.hpp"
#include "solver.hpp"

// ScoreTable stores the scores of all sentences with respect to a solution
typedef vector<double> Scores;
typedef vector<Scores> ScoreTable;

class EvalFunc: public SolEvalFunc
{
  Dict &d1;           // we need to update the sentence scores
  const Dict &d2;

  ScoreTable sTable;

  int minDistance (int w1, int w2, const Sentence &s);

public:

  // 'popSize' is the size of the solution population
  EvalFunc (Dict &d1, const Dict &d2, Solver::size_type popSize);

  // return the sentence ID which has the lowest score
  int operator () (int solID) const
  {
    return min_element(sTable[solID].begin(),sTable[solID].end())
      -sTable[solID].begin();
  }

  // return the stored score - not a real evaluation
  double operator () (int solID, int senID) const { return sTable[solID][senID]; }

  // evaluate 'sol' on all sentences
  double operator () (const Solution &sol);
};

#endif
