// ======================================================================
// misc.cpp - Misc. supporting code for Analyzer main program
// 
// 110900: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 021701: Benjamin Han <benhdj@cs.cmu.edu> Add '-addBest' command-line
//         option.
// ======================================================================

//   Copyright (C) 2000 Benjamin Han <benhdj@cs.cmu.edu>
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "analyzer.hpp"

// remove the leading spaces and convert to lowercase
void processString (string &str)
{
  int i,s;

  str.erase(0,str.find_first_not_of(" \t"));
  for (i=0,s=str.length();i<s;++i) str[i]=tolower(str[i]);
}

void printHelp ()
{
  cerr<<"* analyzer [-backward|-b] [-c1 <int>] [-c2 <int>] [-relaxed|-r 1|2]"<<endl
      <<"           [-addBest|-a] [-numiter|-n <int>] [-solFile|-s]"<<endl
      <<"           [-popsize|-p <int>] [-epochs|-e <int>] [-goal|-g <double>]"<<endl
      <<"           [-crossover|-x <double>] [-mutate|-m <double>] [-creep|-c <double>]"<<endl
      <<"           [-history|-h <int>] [-printInt|-i <int>] [-display|-d]"<<endl
      <<"           corpus-file POST-file"<<endl<<endl;

  cerr<<"  (default values are shown in square brackets; L1 and L2 are two languages)"<<endl;
  cerr<<"  -backward:   if set the translation direction is L2->L1; otherwise L1->L2"<<endl;
  cerr<<"  -c1,-c2:     cutoff values for L1 and L2 [-1, no cutoff]"<<endl;
  cerr<<"  -relaxed:    ignore the binary non-equivalence constraints if it's set"<<endl;
  cerr<<"               mode 2/1 initializes the solutions with/out constraints"<<endl;
  cerr<<"  -addBest:    include the statistically best solution in the init population"<<endl;
  cerr<<"  -numiter:    # of iterations in computing the candidate concepts [2]"<<endl;
  cerr<<"  -solFile:    load "<<SOLFILE<<" as the initial population"<<endl;
  cerr<<"  -popSize:    population size for the genetic algorithm (GA) [100]"<<endl;
  cerr<<"  -epochs:     run until # of epochs; set it <=0 to have no limit [500]"<<endl;
  cerr<<"  -goal:       run until the best solution reaches the goal score [-1, no limit]"<<endl;
  cerr<<"  -crossover:  crossover rate for GA [1/3]"<<endl;
  cerr<<"  -mutate:     mutation rate for GA [1/3]"<<endl;
  cerr<<"  -creep:      creeping rate for GA - set it to 0 under non-relaxed mode [1/3]"<<endl;
  cerr<<"  -history:    operator history window for adaptation [5]"<<endl;
  cerr<<"  -printInt:   every n epochs record the statistics in "<<STATFILE<<" [1]"<<endl;
  cerr<<"  -display:    show dictionary content WITHOUT running GA (all GA parameters"<<endl;
  cerr<<"               ignored)"<<endl<<endl;

  cerr<<"  corpus-file: formatted and tokenized text file with pairs of L2"<<endl;
  cerr<<"               and L1 sentence, in that order; pairs are separated"<<endl;
  cerr<<"               by one or more blank lines; punctuation marks are"<<endl;
  cerr<<"               surrounded by space(s)"<<endl;
  cerr<<"  POST-file:   a part-of-speech tags file for the target language (whether L1"<<endl;
  cerr<<"               or L2 is the target language depends on the translation direction);"<<endl;
  cerr<<"               for English use epost to generate this file"<<endl<<endl;
}

void printStat (const Dict &d1, const Dict &d2)
{
  cout<<"* Total # of sentences = "<<d1.getNumSentences()<<endl;
  cout<<"* Total # of L1        = "<<d1.getNumWords()<<endl;
  cout<<"* Total # of L2        = "<<d2.getNumWords()<<endl<<endl;
}
